# noqa: D100
from atsphinx.mini18n import get_template_dir

extensions = [
    "atsphinx.mini18n",
]

html_sidebars = {"**": [f"{get_template_dir()}/mini18n/snippets/select-lang.html"]}

mini18n_default_language = "en"
mini18n_support_languages = ["en", "ja"]
mini18n_build_style = "nested"



"""Tests for JavaScript layer."""

import http.server
import socket
import threading
import time
from pathlib import Path
from typing import Tuple

import pytest
from playwright.sync_api import Page
from sphinx.testing.util import SphinxTestApp


class ServerOnPytest(http.server.ThreadingHTTPServer):
    """Wrapper class to work using pytest tmpdir and random port."""

    def _is_port_in_use(self, port: int) -> bool:
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            return s.connect_ex(("localhost", port)) == 0

    def __init__(self, directory: Path, port_range: Tuple[int, int] = (49152, 60999)):
        """Initialize server with found usable port.

        Default port-range is ephemeral ports.
        """
        self.directory = directory
        for v in range(port_range[0], port_range[1] + 1):
            if not self._is_port_in_use(v):
                self.port = v
                break
            raise ValueError("All ports are already used.")
        return super().__init__(("", self.port), http.server.SimpleHTTPRequestHandler)

    def finish_request(self, request, client_address):  # noqa: D102
        self.RequestHandlerClass(
            request,
            client_address,
            self,  # type: ignore[invalid-argument-type] - TODO: Detect reason of ty error
            directory=self.directory,  # type: ignore[unknown-argument] - TODO: Detect reason of ty error
        )

    def __enter__(self):  # noqa: D105
        self._thread = threading.Thread(target=self.serve_forever)
        self._thread.daemon = True
        self._thread.start()
        return super().__enter__()

    def __exit__(self, *args, **kwargs):  # noqa: D105
        print(args, kwargs)
        self.shutdown()
        super().__exit__()  # type:ignore[missing-argument] - TODO: Detect reason of ty error


@pytest.mark.sphinx("mini18n-html", testroot="e2e")
def test__root(app: SphinxTestApp, page: Page):
    app.build()
    with ServerOnPytest(app.outdir) as server:  # type: ignore[arg-type]
        url = f"http://localhost:{server.port}/"
        page.goto(url)
        time.sleep(0.5)
        assert page.url.endswith("/en/")


@pytest.mark.sphinx("mini18n-dirhtml", testroot="e2e")
def test__root_dirhtml(app: SphinxTestApp, page: Page):
    app.build()
    with ServerOnPytest(app.outdir) as server:  # type: ignore[arg-type]
        url = f"http://localhost:{server.port}/"
        page.goto(url)
        time.sleep(0.5)
        assert page.url.endswith("/en/")


@pytest.mark.sphinx("mini18n-html", testroot="e2e")
def test__select_language(app: SphinxTestApp, page: Page):
    app.build()
    with ServerOnPytest(app.outdir) as server:  # type: ignore[arg-type]
        url = f"http://localhost:{server.port}/"
        page.goto(url)
        time.sleep(0.5)
        assert page.url.endswith("/en/")
        assert not page.context.cookies()
        element = page.get_by_label("Language:")
        element.select_option(label="ja")
        element.evaluate(
            "elm => elm.dispatchEvent(new Event('change', {bubbles: true}))"
        )
        time.sleep(0.5)
        assert page.url.endswith("/ja/index.html")
        cookies = page.context.cookies()
        assert cookies[0]["name"] == "lang"
        assert cookies[0]["value"] == "ja"
        page.goto(url)
        time.sleep(0.5)
        assert page.url.endswith("/ja/")


@pytest.mark.sphinx("mini18n-html", testroot="e2e")
def test__select_language_sub(app: SphinxTestApp, page: Page):
    app.build()
    with ServerOnPytest(app.outdir) as server:  # type: ignore[arg-type]
        url = f"http://localhost:{server.port}"
        page.goto(f"{url}/en/sub.html")
        time.sleep(0.5)
        element = page.get_by_label("Language:")
        element.select_option(label="ja")
        element.evaluate(
            "elm => elm.dispatchEvent(new Event('change', {bubbles: true}))"
        )
        time.sleep(0.5)
        assert page.url.endswith("/ja/sub.html")


@pytest.mark.sphinx("mini18n-dirhtml", testroot="e2e")
def test__select_language_dirhtml(app: SphinxTestApp, page: Page):
    app.build()
    with ServerOnPytest(app.outdir) as server:  # type: ignore[arg-type]
        url = f"http://localhost:{server.port}/"
        page.goto(url)
        time.sleep(0.5)
        assert page.url.endswith("/en/")
        assert not page.context.cookies()
        element = page.get_by_label("Language:")
        element.select_option(label="ja")
        element.evaluate(
            "elm => elm.dispatchEvent(new Event('change', {bubbles: true}))"
        )
        time.sleep(0.5)
        assert page.url.endswith("/ja/")
        cookies = page.context.cookies()
        assert cookies[0]["name"] == "lang"
        assert cookies[0]["value"] == "ja"
        page.goto(url)
        time.sleep(0.5)
        assert page.url.endswith("/ja/")


@pytest.mark.sphinx("mini18n-dirhtml", testroot="e2e")
def test__select_language_dirhtml_sub(app: SphinxTestApp, page: Page):
    app.build()
    with ServerOnPytest(app.outdir) as server:  # type: ignore[arg-type]
        url = f"http://localhost:{server.port}"
        page.goto(f"{url}/en/sub/")
        time.sleep(0.5)
        element = page.get_by_label("Language:")
        element.select_option(label="ja")
        element.evaluate(
            "elm => elm.dispatchEvent(new Event('change', {bubbles: true}))"
        )
        time.sleep(0.5)
        assert page.url.endswith("/ja/sub/")


@pytest.mark.sphinx(
    "mini18n-dirhtml",
    testroot="e2e-nested",
)
def test__nested_style(app: SphinxTestApp, page: Page):
    app.build()
    with ServerOnPytest(app.outdir) as server:  # type: ignore[arg-type]
        url = f"http://localhost:{server.port}/"
        page.goto(url)
        time.sleep(0.5)
        assert not page.url.endswith("/en/")
        assert not page.context.cookies()
        element = page.get_by_label("Language:")
        element.select_option(label="ja")
        element.evaluate(
            "elm => elm.dispatchEvent(new Event('change', {bubbles: true}))"
        )
        time.sleep(0.5)
        assert page.url.endswith("/ja/")
        element = page.get_by_label("Language:")
        element.select_option(label="en")
        element.evaluate(
            "elm => elm.dispatchEvent(new Event('change', {bubbles: true}))"
        )
        page.wait_for_load_state("networkidle")
        assert page.url == f"http://localhost:{server.port}/"



"""Sphinx builder for i18n site on single deployment."""

from __future__ import annotations

from dataclasses import dataclass, field, fields
from pathlib import Path
from subprocess import PIPE, run
from typing import TYPE_CHECKING, ClassVar, Literal

from jinja2 import Template
from sphinx.builders.dummy import DummyBuilder

if TYPE_CHECKING:
    from docutils import nodes
    from sphinx.application import Sphinx
    from sphinx.config import Config


__version__ = "0.5.1"

package_root = Path(__file__).parent.resolve()


@dataclass
class BuildArgs:
    """Parameter class to build other language."""

    app: Sphinx
    builder: str
    lang: str
    use_lang_dir: bool = True


@dataclass
class ExtensionConfig:
    """Config for this extension from Sphinx config object."""

    PREFIX: ClassVar[str] = "mini18n_"

    build_style: Literal["flat", "nested"] = "flat"
    default_language: str | None = None
    support_languages: list[str] = field(default_factory=list)
    basepath: str = "/"
    select_lang_label: str = "Language:"

    @classmethod
    def from_sphinx(cls, config: Config) -> "ExtensionConfig":
        """Create from Sphinx config object."""
        kv = {
            f.name: getattr(config, f"{cls.PREFIX}{f.name}")
            for f in fields(cls)
            if hasattr(config, f"{cls.PREFIX}{f.name}")
        }
        return cls(**kv)


class Mini18nBuilderBase(DummyBuilder):
    """Abstract bridge builder to build multi-language settings."""

    name_prefix = "mini18n-"

    def finish(self):  # noqa: D102
        config = ExtensionConfig.from_sphinx(self.config)
        if config.build_style == "flat":
            self.finish_tasks.add_task(self.build_heading_content)
        if config.default_language:
            self.finish_tasks.add_task(
                build_i18_contents,
                BuildArgs(
                    self.app,
                    self.name[len(self.name_prefix) :],
                    config.default_language,
                    config.build_style == "flat",
                ),
            )
        for lang in config.support_languages:
            if lang == config.default_language:
                continue
            self.finish_tasks.add_task(
                build_i18_contents,
                BuildArgs(self.app, self.name[len(self.name_prefix) :], lang),
            )

    def build_heading_content(self):
        """Generate root index file."""
        ctx = {
            "config": self.app.config,
        }
        out_template = Path(__file__).parent / "templates" / "mini18n" / "index.html"
        template = Template(out_template.read_text())
        out_index = (
            Path("/".join([self.app.outdir, "index.html"]))
            if isinstance(self.app.outdir, str)
            else self.app.outdir / "index.html"
        )
        Path(out_index).write_text(template.render(ctx))


def build_i18_contents(args: BuildArgs):
    """Run build for coufigured language."""
    if args.use_lang_dir:
        lang_out_dir = (
            "/".join([args.app.outdir, args.lang])
            if isinstance(args.app.outdir, str)
            else args.app.outdir / args.lang  # type: ignore[unsupported-operator]
        )
    else:
        lang_out_dir = args.app.outdir
    cmd = [
        "sphinx-build",
        "-b",
        args.builder,
        "-d",
        args.app.doctreedir,
        args.app.srcdir,
        lang_out_dir,
        "-D",
        f"language={args.lang}",
    ]
    run(cmd, stdout=PIPE, stderr=PIPE)


def register_i18n_builders(app: Sphinx):
    """Create and register dynamic builder for html output."""
    new_builders = []
    for name, Builder in app.registry.builders.items():
        if Builder.format != "html":
            continue

        class Mini18nBuilder(Mini18nBuilderBase):
            pass

        Mini18nBuilder.name = f"mini18n-{name}"

        new_builders.append(Mini18nBuilder)
    for builder_class in new_builders:
        app.add_builder(builder_class)


def autocomplete_config(app: Sphinx, config: Config):
    """Calucrate extension's config values."""
    if not config.mini18n_default_language:
        config.mini18n_default_language = config.language  # type: ignore[attr-defined]
    if not config.mini18n_support_languages:
        config.mini18n_support_languages = [config.mini18n_default_language]  # type: ignore[attr-defined]
    if not config.html_context:
        config.html_context = {}  # type: ignore[attr-defined]
    config.html_context["mini18n"] = {
        "basepath": config.mini18n_basepath,
        "support_languages": config.mini18n_support_languages,
        "select_lang_label": config.mini18n_select_lang_label,
    }


# TODO: Find other api to register function.
def bind_pathto_with_lang(
    app: Sphinx,
    pagename: str,
    templatename: str,
    context: dict,
    doctree: nodes.document,
):
    """Add template functions into context."""
    config = ExtensionConfig.from_sphinx(app.config)

    def pathto_with_lang(otheruri: str, lang: str) -> str:
        """Create path info to specified page and language.

        :param otheruri: Target pagename of document.
        :param lang: Target language that is defined on ``mini18n_support_languages``.
        :returns: URL to page as absolute path.
        """
        if config.build_style == "nested" and lang == config.default_language:
            lang = ""
        else:
            lang = f"{lang}/"
        pathto = app.builder.get_target_uri(otheruri)
        return f"{context['mini18n']['basepath']}{lang}{pathto}"

    context["pathto_with_lang"] = pathto_with_lang
    pass


def get_template_dir() -> str:  # noqa: D103
    return str(package_root / "templates")


def setup(app: Sphinx):  # noqa: D103
    app.add_config_value("mini18n_build_style", "flat", "env")
    app.add_config_value("mini18n_default_language", None, "env")
    app.add_config_value("mini18n_support_languages", [], "env")
    app.add_config_value("mini18n_basepath", "/", "env")
    app.add_config_value("mini18n_select_lang_label", "Language:", "env")
    app.connect("config-inited", autocomplete_config)
    app.connect("html-page-context", bind_pathto_with_lang)

    # IMPORTANT!!
    # This is very dirty hack to work it for any builders of third-party extensions.
    _preload_builder = app.preload_builder

    def preload_builder(buildername):
        """Wrap for :meth:`sphinx.application.Sphinx.preload_builder`."""
        register_i18n_builders(app)
        _preload_builder(buildername)

    app.preload_builder = preload_builder  # type: ignore[method-assign]

    return {
        "version": __version__,
        "env_version": 1,
        "parallel_read_safe": True,
        "parallel_write_safe": True,
    }



#!/usr/bin/env python
"""Wapper to ``age`` CLI.

age cannot copy and move files in updating process.
"""

import argparse
import shutil
from pathlib import Path
from subprocess import run

from atsphinx.mini18n import __version__ as version

project_root = Path(__file__).parent.parent
archive_to = project_root / "doc" / "changelogs"

parser = argparse.ArgumentParser()
parser.add_argument("age_args", nargs="+")  # All arguments are passed to age-cli.


def main(args: argparse.Namespace):  # noqa: D103
    # Move changelog into document to use as old logs.
    changes_rst = project_root / "CHANGES.rst"
    shutil.move(str(changes_rst), archive_to / f"{version}.rst")
    changes_rst.write_text("TEMPLATE")
    # Run age.
    run(["age"] + args.age_args)


if __name__ == "__main__":
    main(parser.parse_args())
